#!/bin/bash

# Script to make a package for Luci from upstream code or from provided local
# tree.
#
# Script supposes that $RPMDEVTREE variable points to the directory structure
# as generated with 'rpmdev-setuptree' command.
#
# v0.1 -- initial version
#
# jpokorny at, redhat, com

PROJECT="luci"
GITURL="git://git.fedorahosted.org/luci.git"
RPMDEVTREE="$HOME/rpmbuild"
TIMESTAMP="$(date +%y%m%d_%H%M)"
TEMP="/tmp"
LOGDIR="$RPMDEVTREE"
PBUILDOPT="-bb"

declare id

function usage ()
{
    echo "Usage: $0 [-c | <luci source path>]"
    echo "When <luci source path> not given, upstream code is used."
    echo "Option '-c' serves for cleaning the (itermediate) products of packaging."
}

function create_exclusion_list ()
{
    # Insert files that should be removed from the code base here.
    # Common wild-cards can be used.
    #
    # Note that the top level dir in the tree is not known,
    # use star ('*') instead.
    echo "Creating list of files to be excluded..."
    cat > "$1" <<EOF
*/.gitignore
*/deploy.sh
*/deploy-devel.sh
EOF
}

function upstream_src ()
{
    id="$TIMESTAMP.upstream"

    # git clone / git pull
    if [ -d "$RPMDEVTREE/SOURCES/$PROJECT" ]; then
        echo "Pulling the current code from upstream..."
        pushd "$(pwd)" >/dev/null
        cd "$RPMDEVTREE/SOURCES/$PROJECT"
        git pull
        popd >/dev/null
    else
        echo "Cloning upstream GIT repository..."
        pushd "$(pwd)" >/dev/null
        cd "$RPMDEVTREE/SOURCES"
        git clone "$GITURL" $PROJECT
        popd >/dev/null
    fi

    # Generate exclusion list file and than convert it into
    # space-separated string.
    exclusion_list="$TEMP/$id"
    create_exclusion_list "$exclusion_list"
    exclude="$(cat $exclusion_list | sed "$!N;s/\n/ /")"
    #echo "$exclude"

    # 'git archive' chosen as requested commit can be parametrized
    # in the future.
    echo "Preparing sources for packaging..."
    pushd "$(pwd)" >/dev/null
    cd "$RPMDEVTREE/SOURCES/$PROJECT"
    cp -p "$PROJECT.spec" "$RPMDEVTREE/SPECS/$PROJECT-$id.spec"
    git archive --format=tar --prefix=$PROJECT-$id/ HEAD >"../$PROJECT-$id.tar"
    tar -f "../$PROJECT-$id.tar" --delete $exclude
    gzip -c "../$PROJECT-$id.tar" >"../$PROJECT-$id.tar.gz"
    rm -f "../$PROJECT-$id.tar"
    popd >/dev/null

    rm -f "$exclusion_list"
}

function own_src ()
{
    id="$TIMESTAMP.ownsrc"

    if [ ! -d "$1" ]; then
        echo "No such directory: $1"
        exit 64
    fi

    exclusion_list="$TEMP/$id"
    create_exclusion_list "$exclusion_list"

    # Not 'git archive' as it is not presumed that provided
    # Luci source path is a git repository (or its clone).
    echo "Preparing sources for packaging..."
    cp -p "$1/$PROJECT.spec" "$RPMDEVTREE/SPECS/$PROJECT-$id.spec"
    tar -czpf "$RPMDEVTREE/SOURCES/$PROJECT-$id.tar.gz" --exclude-vcs --exclude-from="$exclusion_list" --transform="s,^,$PROJECT-$id/," -C "$1" .

    rm -f "$exclusion_list"
}

function make_pkg ()
{
    echo "Preparing SPEC file..."
    sed -i.bck \
        -e "s/^\(Version:[ ]*\)\([^$]*\)$/\1$id\n%global origversion \2/" \
        -e "s/^\(Source0:[ ]*\)[^$]*$/\1$PROJECT-$id.tar.gz/" \
        -e "s/%{version}/%{origversion}/" \
        "$RPMDEVTREE/SPECS/$PROJECT-$id.spec"

    echo "Packaging (using $PBUILDOPT option)..."
    rpmbuild "$PBUILDOPT" "$RPMDEVTREE/SPECS/$PROJECT-$id.spec" &>"$LOGDIR/$PROJECT-$id.log"
    echo "Log from this packaging process stored in $LOGDIR/$PROJECT-$id.log"
}

function clean_rpmdevtree ()
{
    echo "Cleaning files and directories in $RPMDEVTREE and also logs in $LOGDIR..."
    rm -rf "$RPMDEVTREE/BUILD/$PROJECT"-* \
           "$RPMDEVTREE/BUILDROOT/$PROJECT"-*
    rm -f "$RPMDEVTREE/RPMS/x86_64/$PROJECT"-*.rpm \
          "$RPMDEVTREE/SOURCES/$PROJECT"-*.tar.gz \
          "$RPMDEVTREE/SPECS/$PROJECT"-* \
          "$RPMDEVTREE/SRPMS/$PROJECT"-*.rpm \
          "$LOGDIR/$PROJECT"-*.log
}

### main

if [ $# -eq 0 ]; then
    upstream_src
elif [ "$1" == "-h" -o "$1" == "--help" ]; then
    usage
    exit 0
elif [ "$1" == "-c" -o "$1" == "--clean" ]; then
    clean_rpmdevtree
    exit 0
else
    own_src $1
fi
make_pkg
